
/* =================================================== GENERAL SIMPLIFICATION */

simplify(X,X):-
   atomic(X).
simplify(X1,Y):-
   specific_simplification(X1,X),
   X =.. [Name|Params],
   simplify_params(Params,SimplifiedParams),
   Specific =.. [Name|SimplifiedParams],

   (forall(member(P,SimplifiedParams), (integer(P);float(P)))
   -> Y is Specific
   ;  specific_simplification(Specific, Temp),
      (Temp \= Specific
      -> simplify(Temp,Y)
      ;  Y = Temp)
   ).
simplify_params([],[]).
simplify_params([Param|Rest],[Simplified|SimplifiedRest]):-
   simplify(Param, Simplified),
   simplify_params(Rest, SimplifiedRest).

/* ================================================================= ADDITION */

specific_simplification(A+0, A).
specific_simplification(0+B, B).

specific_simplification(A+(-A), 0).
specific_simplification((-A)+A, 0).
specific_simplification(A+A, 2*A).

specific_simplification(A+B*A, (B+1)*A).
specific_simplification(A+A*B, (B+1)*A).
specific_simplification(A*B+A, (B+1)*A).
specific_simplification(B*A+A, (B+1)*A).
specific_simplification(C*A+B*A, (C+B)*A).
specific_simplification(A*C+B*A, (C+B)*A).
specific_simplification(C*A+A*B, (C+B)*A).
specific_simplification(A*C+A*B, (C+B)*A).

/* ============================================================== SUBTRACTION */

specific_simplification(A-0, A).
specific_simplification(0-B, (-B)).

specific_simplification(A-A, 0).
specific_simplification((-A)-A, -(2*A)).
specific_simplification(A-(-A), 2*A).

specific_simplification(A-B*A, (1-B)*A).
specific_simplification(A-A*B, (1-B)*A).
specific_simplification(A*B-A, (B-1)*A).
specific_simplification(B*A-A, (B-1)*A).
specific_simplification(C*A-B*A, (C-B)*A).
specific_simplification(A*C-B*A, (C-B)*A).
specific_simplification(C*A-A*B, (C-B)*A).
specific_simplification(A*C-A*B, (C-B)*A).

specific_simplification(A-(B-C), (A-B)+C).

/* =========================================================== MULTIPLICATION */

specific_simplification(_*0, 0).
specific_simplification(0*_, 0).
specific_simplification(A*1, A).
specific_simplification(1*A, A).

specific_simplification(A*(B/A),B).
specific_simplification((B/A)*A,B).

specific_simplification(A*A^B,   A^(B+1)).
specific_simplification(A^B*A,   A^(B+1)).
specific_simplification(A^C*A^B, A^(C+B)).

specific_simplification((A/B)*(C/D), (A*C)/(B*D)).

/* ================================================================= DIVISION */

specific_simplification(0/0, _):-fail.
specific_simplification(0/_, 0).
specific_simplification(A/A, 1).
specific_simplification(A/1, A).

specific_simplification(A/(B/C), (A*C)/B).
specific_simplification((A/B)/C, A/(B*C)).
specific_simplification((A/B)/(C/D), (A*D)/(B*C)).

/* ==================================================================== POWER */

specific_simplification(0^0, _):-fail.
specific_simplification(_^0, 1).
specific_simplification(exp(0),1).
specific_simplification(0^_, 0).

specific_simplification(A^(-B), 1/A^B).
specific_simplification((A^B)^C, A^(B*C)).
specific_simplification(A^(B/C), sqrt(C, A^B)).

/* =============================================== NO SIMPLIFICATION POSSIBLE */

specific_simplification(X,X).

/* ==================================================================== TOOLS */

:-arithmetic_function(ln/1).
:-arithmetic_function(log/2).
ln(X,Y):-
   log(X,Y).
log(A,X,Y):-
   Y is log(X)/log(A).
