
/* =================================================== */
/*                        Input                        */
/* =================================================== */

removeWhiteSpaces([],[]):-!.
removeWhiteSpaces([9|Rest],Result):-  /* TAB   */
   removeWhiteSpaces(Rest,Result),!.
removeWhiteSpaces([32|Rest],Result):- /* SPACE */
   removeWhiteSpaces(Rest,Result),!.
removeWhiteSpaces([First|Rest],[First|ResultRest]):-
   removeWhiteSpaces(Rest,ResultRest).

signature([45|Rest],Rest,-1):-!.
signature([43|Rest],Rest,1):-!.
signature(X,X,1).

/* =================================================== */

readString([]):-
   peek_char(10),
   get0(10),
   !.
readString([First|Rest]):-
   get0(First),
   readString(Rest).

/* =================================================== */

strToInt([],1,0).
strToInt([First|Rest],NextPower,Int):-
   integer(First),
   48 =< First,
   57 >= First,
   strToInt(Rest,OwnPower,RestInt),
   Int is RestInt + OwnPower * (First-48),
   NextPower is OwnPower*10.

readInteger(X):-
   readString(Str),
   removeWhiteSpaces(Str,TrimmedStr),
   signature(TrimmedStr,PositiveStr,Sign),
   strToInt(PositiveStr,_,X1),
   X is X1*Sign.

/* =================================================== */

fractionalPart([],_,0).
fractionalPart([First|Rest],OwnPower,Result):-
   integer(First),
   48 =< First,
   57 >= First,
   NextPower is OwnPower / 10,
   fractionalPart(Rest,NextPower,RestResult),
   Result is RestResult+First*OwnPower.

strToFloat([],1,0).                               /* no fractional part */
strToFloat([46|Fractional],1,Result):-          /* , */
   fractionalPart(Fractional,0.1,Result).
strToFloat([44|Fractional],1,Result):-          /* . */
   fractionalPart(Fractional,0.1,Result).
strToFloat([First|Rest],NextPower,Result):-   /* still integral part */
   integer(First),
   48 =< First,
   57 >= First,
   strToFloat(Rest,OwnPower,RestResult),
   Result is RestResult + First*OwnPower,
   NextPower is OwnPower*10.

readFloat(X):-
   readString(Str),
   removeWhiteSpaces(Str,TrimmedStr),
   signature(TrimmedStr,PositiveStr,Sign),
   strToFloat(PositiveStr,_,X1),
   X is X1*Sign.

/* =================================================== */
/*                       Output                        */
/* =================================================== */

writeString([]).
writeString([First|Rest]):-
   char_code(FirstChar,First),
   write(FirstChar),
   writeString(Rest).



writeMultiple(Count,_):-
   integer(Count),
   Count =< 0,!.
writeMultiple(Count,Term):-
   integer(Count),
   Count1 is Count-1,
   writeMultiple(Count1,Term),
   write(Term).

intLength(Int,1):-
   Int >= 0,
   Int =< 9.
intLength(Int,Length):-
   Int >= 10,
   Int2 is Int // 10,
   intLength(Int2,Length2),
   Length is Length2+1.
intLength(Int,Length):-
   Int < 0,
   Int2 is (-1)*Int,
   intLength(Int2,Length2),
   Length is Length2+1.

writeInteger(Int, MinChars):-
   nonvar(MinChars),
   intLength(Int,Length),
   SpaceCount is MinChars-Length,
   writeMultiple(SpaceCount,' '),
   write(Int),
   !.
writeInteger(Int, _):-
   writeInteger(Int,0).



fractionalLength(0.0, _, 0):-!.
fractionalLength(0, _, 0):-!.
fractionalLength(_, 0, 0):-!.
fractionalLength(Fractional, MaxFractional, Length):-
   write(Fractional),write('   '),
   Test is truncate(Fractional*10),
   write(Test),write('   '),
   Fractional2 is Fractional*10-truncate(Fractional*10),
   write(Fractional2),nl,
   MaxFractional2 is MaxFractional-1,
   fractionalLength(Fractional2,MaxFractional2,Length2),
   Length is Length2+1.

writeFloat(Float,MinChars,MinFractional,MaxFractional):-
   Fractional is float_fractional_part(Float),
   Integral is float_integer_part(Float),
   intLength(Integral,ILength),
   fractionalLength(Fractional,MaxFractional).





