

greek(alpha,  '\\alpha').
greek(beta,   '\\beta').
greek(gamma,  '\\gamma').
greek(delta,  '\\delta').
greek(epsilon,'\\epsilon').
greek(zeta,   '\\zeta').
greek(eta,    '\\eta').
greek(theta,  '\\theta').
greek(iota,   '\\iota').
greek(kappa,  '\\kappa').
greek(lambda, '\\lambda').
greek(my,     '\\my').
greek(ny,     '\\ny').
greek(xi,     '\\xi').
greek(omikron,'\\omikron').
greek(pi,     '\\pi').
greek(rho,    '\\rho').
greek(sigma,  '\\sigma').
greek(tau,    '\\tau').
greek(ypsilon,'\\ypsilon').
greek(phi,    '\\varphi').
greek(chi,    '\\chi').
greek(psi,    '\\psi').
greek(omega,  '\\omega').




/*  ===============  ATOMICS  ===============  */

latexer(A):-
   atomic(A),
   greek(A,G),
   write(G),
   !.
latexer(A):-
   atomic(A),
   write(A),
   !.
   
   
   
   
   /*  ===============  OPERATORS  ===============  */

latexer(A+B):-
   latexer(A),
   write('+'),
   latexer(B),
   !.

latexer(A-B):-
   latexer(A),
   write('-'),
   latexer(B),
   !.
   
latexer(A*B):-
   latexer(A),
   write(' \\cdot '),
   latexer(B),
   !.

latexer(A/B):-
   write(' \\frac{'),
   latexer(A),
   write('}{'),
   latexer(B),
   write('} '),
   !.




   /*  ===============  POWERS,ROOTS,LOGARITHM  ===============  */

latexer(A^B):-
   latexer(A),
   write('^{'),
   latexer(B),
   write('} '),
   !.
latexer(A^(B)):-
   latexer(A^B),
   !.
   
latexer(sqrt(A)):-
   write(' \\sqrt{'),
   latexer(A),
   write('} '),
   !.

latexer(sqrt(A,B)):-
   write(' \\sqrt['),
   latexer(A),
   write(']{'),
   latexer(B),
   write('} '),
   !.

latexer(log(A,B)):-
   write(' \\log_{'),
   latexer(A),
   write('}\\left( '),
   latexer(B),
   write(' \\right) '),
   !.

latexer(ln(A)):-
   write(' ln\\left( '),
   latexer(A),
   write(' \\right) '),
   !.
   
   

   
/*  ===============  SUM,INTEGRAL  ===============  */
   
latexer(sum(A,B,C)):-
   write(' \\sum\\limits_{'),
   latexer(A),
   write('}^{'),
   latexer(B),
   write('}\\left( '),
   latexer(C),
   write(' \\right) '),
   !.
   
latexer(integral(A,B,C,D)):- /* D is for example dx */
   write(' \\int\\limits_{'),
   latexer(A),
   write('}^{'),
   latexer(B),
   write('} '),
   latexer(C),
   latexer(D),
   !.
   



/*  ===============  RELATORS  ===============  */
   
latexer(A =< B):-
   latexer(A),
   write(' \\le '),
   latexer(B),
   !.
latexer(A < B):-
   latexer(A),
   write(' < '),
   latexer(B),
   !.
latexer(A >= B):-
   latexer(A),
   write(' \\ge '),
   latexer(B),
   !.
latexer(A > B):-
   latexer(A),
   write(' > '),
   latexer(B),
   !.
latexer(A == B):-
   latexer(A),
   write(' = '),
   latexer(B),
   !.
latexer(A = B):-
   latexer(A),
   write(' = '),
   latexer(B),
   !.
latexer(A =\= B):-
   latexer(A),
   write(' \\not= '),
   latexer(B),
   !.



   
/*  ===============  MATRICES  ===============  */

latexer(matrix([A|Rest])):- /* A is the first line of the Matrix */
   length(A,Width),
   allLinesHaveLength(Rest,Width),

   write(' \\left( \\begin{array}{'),
   writeMultiple(Width,'c'),
   write('}'),
   writeMatrix([A|Rest]),
   write('\\end{array} \\right) ').




   /*  ===============  PARANTHESIS  ===============  */
   
latexer(A):-
   (B) = A,
   write(' \\left( '),
   latexer(B),
   write(' \\right) '),
   !.
 
  
latexer([A]):- /* alright, this is not a parenthesis, its a list */
   write(' \\left[ '), /* but the user wont know this hehehehehe */
   latexer(A),
   write(' \\right] '),
   !.




/*  ===============  MAIN  ===============  */
   
latex(Term):-
   write('   \\[     '),
   latexer(Term),
   write('     \\]'), nl.

main:-
   read(Terms),
   nl,nl,nl,nl,
   write('\\documentclass[10pt]{article}'), nl,
   write('\\usepackage{amsmath}'), nl, nl,
   write('\\begin{document}'), nl,
   latex(Terms),
   write('\\end{document}'), nl, nl.
   
   
   
   
   
   
   
   
   


length([],0).
length([_|R],N):-
   length(R,N1),
   N is N1+1.
allLinesHaveLength([],_).
allLinesHaveLength([A|R],L):-
   length(A,L),
   allLinesHaveLength(R,L).

writeMultiple(0,_):-!.
writeMultiple(N,Term):-
   write(Term),
   N1 is N-1,
   writeMultiple(N1,Term).

writeMatrixLine([A],true):-
   latexer(A),
   write('\\\\'),!.
writeMatrixLine([A],false):-
   latexer(A).
writeMatrixLine([A|Rest], Last):-
   latexer(A),
   write(' & '),
   writeMatrixLine(Rest, Last).
   
writeMatrix([A]):-
   writeMatrixLine(A,false),
   !.
writeMatrix([A|R]):-
   writeMatrixLine(A,true),
   writeMatrix(R).  
